@@ -355,7 +355,11 @@ static unsigned char broke[] =
 static char        *
 _SplitID(char *file)
 {
+#ifndef __EMX__
   char               *p = strrchr(file, ':');
+#else
+  char               *p = strrchr(file, ';');
+#endif
 
   if (p == NULL)
     return "";
@@ -2285,7 +2289,11 @@ Imlib_load_image(ImlibData * id, char *file)
       if (!strcmp(file,"-"))
         p = stdin;
       else
-        p = fopen(fil, "r");
+#ifndef __EMX__
+        p = fopen(file, "r");
+#else
+        p = fopen(file, "rt");
+#endif
       if (!p)
 	{
 	  free(im);
@@ -2339,7 +2347,11 @@ Imlib_load_image(ImlibData * id, char *file)
       fclose(p);
       if (iden[0])
 	{
+#ifndef __EMX__
 	  strncat(fil, ":", sizeof(fil) - strlen(fil));
+#else
+	  strncat(fil, ";", sizeof(fil) - strlen(fil));
+#endif
 	  strncat(fil, iden, sizeof(fil) - strlen(fil));
 	}
     }
@@ -2455,7 +2467,11 @@ Imlib_save_image_to_ppm(ImlibData * id, ImlibImage * im, char *file)
 
   if ((!id) || (!im) || (!file))
     return 0;
+#ifndef __EMX__
   f = fopen(file, "w");
+#else
+  f = fopen(file, "wb");
+#endif
   if (!f)
     return 0;
 
