@@ -4,6 +4,10 @@
 #include "Imlib_private.h"
 #include <locale.h>
 
+#ifdef __EMX__
+extern const char *__XOS2RedirRoot(const char *);
+#endif
+
 #ifndef HAVE_SNPRINTF
 #define snprintf my_snprintf
 #ifdef HAVE_STDARGS
@@ -86,8 +90,8 @@ Imlib_init(Display * disp)
   int                 newcm;
   char               *old_locale;
 
-  fprintf(stderr, "Imlib Init\n");
-  fflush(stderr);
+  /* fprintf(stderr, "Imlib Init\n");
+  fflush(stderr); */
 
   palfile = NULL;
   if (!disp)
@@ -183,9 +187,17 @@ Imlib_init(Display * disp)
 
   old_locale = strdup(setlocale(LC_NUMERIC, NULL));
   setlocale(LC_NUMERIC, "C");
+#ifndef __EMX__
   f = fopen(s, "r");
+#else
+  f = fopen(s, "rt");
+#endif
   if (!f)
+#ifndef __EMX__
     f = fopen(SYSTEM_IMRC, "r");
+#else
+    f = fopen(__XOS2RedirRoot(SYSTEM_IMRC), "rt");
+#endif
   if (f)
     {
       while (fgets(s, 4096, f))
@@ -629,7 +641,7 @@ Imlib_init_with_params(Display * disp, ImlibInitParams * p)
   char               *palfile;
   int                 loadpal;
   int                 vis;
-  int                 newcm;
+  int                 newcm = 0;
   char               *old_locale;
 
   palfile = NULL;
@@ -725,9 +737,17 @@ Imlib_init_with_params(Display * disp, ImlibInitParams * p)
   snprintf(s, sizeof(s), "%s/.imrc", homedir);
   old_locale = strdup(setlocale(LC_NUMERIC, NULL));
   setlocale(LC_NUMERIC, "C");
+#ifndef __EMX__
   f = fopen(s, "r");
+#else
+  f = fopen(s, "rt");
+#endif
   if (!f)
+#ifndef __EMX__
     f = fopen(SYSTEM_IMRC, "r");
+#else
+    f = fopen(__XOS2RedirRoot(SYSTEM_IMRC), "rt");
+#endif
   if (f)
     {
       while (fgets(s, 4096, f))
@@ -935,6 +955,11 @@ Imlib_init_with_params(Display * disp, ImlibInitParams * p)
 	id->cache.size_image = p->imagecachesize;
       if (p->flags & PARAMS_PIXMAPCACHESIZE)
 	id->cache.size_pixmap = p->pixmapcachesize;
+      if (p->flags & PARAMS_COLORMAP)
+	{
+	  id->x.root_cmap = p->cmap;
+	  newcm = 1;
+	}
     }
   /* list all visuals for the default screen */
   xvi.screen = id->x.screen;
@@ -1093,7 +1118,6 @@ Imlib_init_with_params(Display * disp, ImlibInitParams * p)
     at.override_redirect = True;
     mask = CWOverrideRedirect | CWBackPixel | CWBorderPixel |
       CWBackingStore | CWSaveUnder;
-    newcm = 0;
     if (id->x.visual != DefaultVisual(disp, id->x.screen))
       {
 	Colormap            cm;
@@ -1108,6 +1132,11 @@ Imlib_init_with_params(Display * disp, ImlibInitParams * p)
 	    newcm = 1;
 	  }
       }
+    else if (newcm)
+      {
+	mask |= CWColormap;
+	at.colormap = id->x.root_cmap;
+      }
     id->x.base_window = XCreateWindow(id->x.disp, id->x.root,
 				      -100, -100, 10, 10, 0,
 				      id->x.depth, InputOutput,
@@ -1198,7 +1227,8 @@ Imlib_copy_image(ImlibData * id, ImlibImage * im)
   if (!im || !im->pixmap)
     return 0;
   p = XCreatePixmap(id->x.disp, id->x.base_window, im->width, im->height, id->x.depth);
-  tgc = XCreateGC(id->x.disp, p, (unsigned long)0, &gcv);
+  gcv.graphics_exposures = False;
+  tgc = XCreateGC(id->x.disp, p, GCGraphicsExposures, &gcv);
   XCopyArea(id->x.disp, im->pixmap, p, tgc, 0, 0, im->width, im->height, 0, 0);
   XFreeGC(id->x.disp, tgc);
   return p;
@@ -1226,7 +1256,8 @@ Imlib_copy_mask(ImlibData * id, ImlibImage * im)
   if (!im || !im->shape_mask)
     return 0;
   p = XCreatePixmap(id->x.disp, id->x.base_window, im->width, im->height, 1);
-  tgc = XCreateGC(id->x.disp, p, (unsigned long)0, &gcv);
+  gcv.graphics_exposures = False;
+  tgc = XCreateGC(id->x.disp, p, GCGraphicsExposures, &gcv);
   XCopyArea(id->x.disp, im->shape_mask, p, tgc, 0, 0, im->width, im->height, 0, 0);
   XFreeGC(id->x.disp, tgc);
   return p;
@@ -1333,11 +1364,13 @@ Imlib_set_image_shape(ImlibData * id, ImlibImage * im, ImlibColor * color)
 {
   if ((!im) || (!color))
     return;
-
-  im->shape_color.r = color->r;
-  im->shape_color.g = color->g;
-  im->shape_color.b = color->b;
-  dirty_pixmaps(id, im);
+  if ((im->shape_color.r != color->r) || (im->shape_color.g != color->g) || (im->shape_color.b != color->b))
+    {
+      im->shape_color.r = color->r;
+      im->shape_color.g = color->g;
+      im->shape_color.b = color->b;
+      dirty_pixmaps(id, im);
+    }
 }
 
 int
@@ -1375,5 +1408,9 @@ Imlib_get_colormap(ImlibData * id)
 char               *
 Imlib_get_sysconfig(ImlibData * id)
 {
+#ifndef __EMX__
   return strdup(SYSTEM_IMRC);
+#else
+  return strdup(__XOS2RedirRoot(SYSTEM_IMRC));
+#endif
 }
