@@ -3,6 +3,11 @@
 #include "Imlib.h"
 #include "Imlib_private.h"
 
+#ifdef __EMX__
+#include <io.h>
+#include <process.h>
+#endif
+
 #ifndef HAVE_SNPRINTF
 #define snprintf my_snprintf
 #ifdef HAVE_STDARGS
@@ -143,122 +148,152 @@ Imlib_set_image_modifier(ImlibData * id, ImlibImage * im, ImlibColorModifier * m
 {
   if ((!im) | (!mod))
     return;
-  if ((im->mod.gamma != mod->gamma) || (im->mod.brightness != mod->brightness) || (im->mod.contrast != mod->contrast))
-    {
-      im->mod.gamma = mod->gamma;
-      im->mod.brightness = mod->brightness;
-      im->mod.contrast = mod->contrast;
-      calc_map_tables(id, im);
-    }
+   if ((im->mod.gamma != mod->gamma) || (im->mod.brightness != mod->brightness) || (im->mod.contrast != mod->contrast))
+     {
+	im->mod.gamma = mod->gamma;
+	im->mod.brightness = mod->brightness;
+	im->mod.contrast = mod->contrast;
+	calc_map_tables(id, im);
+	if (im->pixmap)
+	  {
+	     free_pixmappmap(id, im->pixmap);
+	     im->pixmap = 0;
+	  }
+	dirty_pixmaps(id, im);
+     }
 }
 
 void
 Imlib_set_image_red_modifier(ImlibData * id, ImlibImage * im, ImlibColorModifier * mod)
 {
-  if ((!im) | (!mod))
-    return;
-  if ((im->rmod.gamma != mod->gamma) || (im->rmod.brightness != mod->brightness) || (im->rmod.contrast != mod->contrast))
-    {
-      im->rmod.gamma = mod->gamma;
-      im->rmod.brightness = mod->brightness;
-      im->rmod.contrast = mod->contrast;
-      calc_map_tables(id, im);
-    }
+   if ((!im) | (!mod))
+      return;
+   if ((im->rmod.gamma != mod->gamma) || (im->rmod.brightness != mod->brightness) || (im->rmod.contrast != mod->contrast))
+     {
+	im->rmod.gamma = mod->gamma;
+	im->rmod.brightness = mod->brightness;
+	im->rmod.contrast = mod->contrast;
+	calc_map_tables(id, im);
+	if (im->pixmap)
+	  {
+	     free_pixmappmap(id, im->pixmap);
+	     im->pixmap = 0;
+	  }
+	dirty_pixmaps(id, im);
+     }
 }
 
 void
 Imlib_set_image_green_modifier(ImlibData * id, ImlibImage * im, ImlibColorModifier * mod)
 {
-  if ((!im) | (!mod))
-    return;
-  if ((im->gmod.gamma != mod->gamma) || (im->gmod.brightness != mod->brightness) || (im->gmod.contrast != mod->contrast))
-    {
-      im->gmod.gamma = mod->gamma;
-      im->gmod.brightness = mod->brightness;
-      im->gmod.contrast = mod->contrast;
-      calc_map_tables(id, im);
-    }
+   if ((!im) | (!mod))
+      return;
+   if ((im->gmod.gamma != mod->gamma) || (im->gmod.brightness != mod->brightness) || (im->gmod.contrast != mod->contrast))
+     {
+	im->gmod.gamma = mod->gamma;
+	im->gmod.brightness = mod->brightness;
+	im->gmod.contrast = mod->contrast;
+	calc_map_tables(id, im);
+	if (im->pixmap)
+	  {
+	     free_pixmappmap(id, im->pixmap);
+	     im->pixmap = 0;
+	  }
+	dirty_pixmaps(id, im);
+     }
 }
 
 void
 Imlib_set_image_blue_modifier(ImlibData * id, ImlibImage * im, ImlibColorModifier * mod)
 {
-  if ((!im) | (!mod))
-    return;
-  if ((im->bmod.gamma != mod->gamma) || (im->bmod.brightness != mod->brightness) || (im->bmod.contrast != mod->contrast))
-    {
-      im->bmod.gamma = mod->gamma;
-      im->bmod.brightness = mod->brightness;
-      im->bmod.contrast = mod->contrast;
-      calc_map_tables(id, im);
-    }
+   if ((!im) | (!mod))
+      return;
+   if ((im->bmod.gamma != mod->gamma) || (im->bmod.brightness != mod->brightness) || (im->bmod.contrast != mod->contrast))
+     {
+	im->bmod.gamma = mod->gamma;
+	im->bmod.brightness = mod->brightness;
+	im->bmod.contrast = mod->contrast;
+	calc_map_tables(id, im);
+	if (im->pixmap)
+	  {
+	     free_pixmappmap(id, im->pixmap);
+	     im->pixmap = 0;
+	  }
+	dirty_pixmaps(id, im);
+     }
 }
 
 void
 Imlib_get_image_modifier(ImlibData * id, ImlibImage * im, ImlibColorModifier * mod)
 {
-  if ((!im) | (!mod))
-    return;
-  mod->gamma = im->mod.gamma;
-  mod->brightness = im->mod.brightness;
-  mod->contrast = im->mod.contrast;
-  calc_map_tables(id, im);
+   if ((!im) | (!mod))
+      return;
+   mod->gamma = im->mod.gamma;
+   mod->brightness = im->mod.brightness;
+   mod->contrast = im->mod.contrast;
+   calc_map_tables(id, im);
 }
 
 void
 Imlib_get_image_red_modifier(ImlibData * id, ImlibImage * im, ImlibColorModifier * mod)
 {
-  if ((!im) | (!mod))
-    return;
-  mod->gamma = im->rmod.gamma;
-  mod->brightness = im->rmod.brightness;
-  mod->contrast = im->rmod.contrast;
+   if ((!im) | (!mod))
+      return;
+   mod->gamma = im->rmod.gamma;
+   mod->brightness = im->rmod.brightness;
+   mod->contrast = im->rmod.contrast;
 }
 
 void
 Imlib_get_image_green_modifier(ImlibData * id, ImlibImage * im, ImlibColorModifier * mod)
 {
-  if ((!im) | (!mod))
-    return;
-  mod->gamma = im->gmod.gamma;
-  mod->brightness = im->gmod.brightness;
-  mod->contrast = im->gmod.contrast;
+   if ((!im) | (!mod))
+      return;
+   mod->gamma = im->gmod.gamma;
+   mod->brightness = im->gmod.brightness;
+   mod->contrast = im->gmod.contrast;
 }
 
 void
 Imlib_get_image_blue_modifier(ImlibData * id, ImlibImage * im, ImlibColorModifier * mod)
 {
-  if ((!im) | (!mod))
-    return;
-  mod->gamma = im->bmod.gamma;
-  mod->brightness = im->bmod.brightness;
-  mod->contrast = im->bmod.contrast;
+   if ((!im) | (!mod))
+      return;
+   mod->gamma = im->bmod.gamma;
+   mod->brightness = im->bmod.brightness;
+   mod->contrast = im->bmod.contrast;
 }
 
 void
 Imlib_set_image_red_curve(ImlibData * id, ImlibImage * im, unsigned char *mod)
 {
-  int                 i;
-  char                same = 1;
-
-  if ((!im) || (!mod))
-    return;
-
-  for (i = 0; i < 256; i++)
-    {
-      if (im->rmap[i] != mod[i])
-	{
-	  same = 0;
-	  break;
-	}
-    }
-  if (same)
-    return;
-
-  for (i = 0; i < 256; i++)
-    im->rmap[i] = mod[i];
-  dirty_pixmaps(id, im);
-  im->mod.contrast = 257;
+   int                 i;
+   char                same = 1;
+   
+   if ((!im) || (!mod))
+      return;
+   
+   for (i = 0; i < 256; i++)
+     {
+	if (im->rmap[i] != mod[i])
+	  {
+	     same = 0;
+	     break;
+	  }
+     }
+   if (same)
+      return;
+   
+   if (im->pixmap)
+     {
+	free_pixmappmap(id, im->pixmap);
+	im->pixmap = 0;
+     }
+   dirty_pixmaps(id, im);
+   for (i = 0; i < 256; i++)
+      im->rmap[i] = mod[i];
+   
+   im->mod.contrast = 257;
 }
 
 void
@@ -281,9 +316,15 @@ Imlib_set_image_green_curve(ImlibData * id, ImlibImage * im, unsigned char *mod)
   if (same)
     return;
 
+   if (im->pixmap)
+     {
+	free_pixmappmap(id, im->pixmap);
+	im->pixmap = 0;
+     }
+   dirty_pixmaps(id, im);
   for (i = 0; i < 256; i++)
     im->gmap[i] = mod[i];
-  dirty_pixmaps(id, im);
+
   im->mod.contrast = 257;
 }
 
@@ -307,10 +348,16 @@ Imlib_set_image_blue_curve(ImlibData * id, ImlibImage * im, unsigned char *mod)
   if (same)
     return;
 
+   if (im->pixmap)
+     {
+	free_pixmappmap(id, im->pixmap);
+	im->pixmap = 0;
+     }
+   dirty_pixmaps(id, im);
   for (i = 0; i < 256; i++)
     im->bmap[i] = mod[i];
-  dirty_pixmaps(id, im);
-  im->mod.contrast = 257;
+
+   im->mod.contrast = 257;
 }
 
 void
@@ -379,9 +426,14 @@ Imlib_apply_modifiers_to_rgb(ImlibData * id, ImlibImage * im)
   im->bmod.gamma = 256;
   im->bmod.brightness = 256;
   im->bmod.contrast = 256;
+   if (im->pixmap)
+     {
+	free_pixmappmap(id, im->pixmap);
+	im->pixmap = 0;
+     }
+   dirty_pixmaps(id, im);
   calc_map_tables(id, im);
   dirty_images(id, im);
-  dirty_pixmaps(id, im);
 }
 
 #define SHADE_PIXEL(pixel, dir, tmp) do {(tmp) = ((((double)pixel)/256.0) + ((dir) ? 0.2 : -0.2)) * 256.0; \
@@ -848,10 +900,16 @@ Imlib_crop_and_clone_image(ImlibData * id, ImlibImage * im, int x, int y, int w,
 void
 Imlib_changed_image(ImlibData * id, ImlibImage * im)
 {
-  if (!im)
-    return;
-  dirty_images(id, im);
-  dirty_pixmaps(id, im);
+   if (!im)
+      return;
+   dirty_images(id, im);
+   if (im->pixmap)
+     {
+	free_pixmappmap(id, im->pixmap);
+	im->pixmap = 0;
+	im->shape_mask = 0;
+     }
+   dirty_pixmaps(id, im);
 }
 
 void
@@ -985,7 +1043,12 @@ Imlib_flip_image_horizontal(ImlibData * id, ImlibImage * im)
   im->border.left = im->border.right;
   im->border.right = w3;
   dirty_images(id, im);
-  dirty_pixmaps(id, im);
+   if (im->pixmap)
+     {
+	free_pixmappmap(id, im->pixmap);
+	im->pixmap = 0;
+     }
+   dirty_pixmaps(id, im);
 }
 
 void
@@ -1022,7 +1085,12 @@ Imlib_flip_image_vertical(ImlibData * id, ImlibImage * im)
   im->border.top = im->border.bottom;
   im->border.bottom = w3;
   dirty_images(id, im);
-  dirty_pixmaps(id, im);
+   if (im->pixmap)
+     {
+	free_pixmappmap(id, im->pixmap);
+	im->pixmap = 0;
+     }
+   dirty_pixmaps(id, im);
 }
 
 void
@@ -1063,7 +1131,12 @@ Imlib_rotate_image(ImlibData * id, ImlibImage * im, int d)
   im->border.bottom = im->border.right;
   im->border.right = w3;
   dirty_images(id, im);
-  dirty_pixmaps(id, im);
+   if (im->pixmap)
+     {
+	free_pixmappmap(id, im->pixmap);
+	im->pixmap = 0;
+     }
+   dirty_pixmaps(id, im);
 }
 
 ImlibImage         *
@@ -2091,8 +2164,12 @@ open_helper(const char *instring, const char *fn, const char *mode)
 	vec[vn] = strdup(fn);
       else if (strncmp(pp, "%P/", 3) == 0)
 	{
+#ifndef __EMX__
 	  strcpy(buf, NETPBM_PATH);
 	  strcat(buf, pp + 2);
+#else
+	  strcpy(buf, pp + 3);
+#endif
 	  if ((vec[vn] = strdup(buf)) == NULL)
 	    break;
 	}
@@ -2108,8 +2185,12 @@ open_helper(const char *instring, const char *fn, const char *mode)
 	}
       else if (strncmp(pp, "%C/", 3) == 0)
 	{
+#ifndef __EMX__
 	  strcpy(buf, CONVERT_PATH);
 	  strcat(buf, pp + 2);
+#else
+	  strcpy(buf, pp + 3);
+#endif
 	  if ((vec[vn] = strdup(buf)) == NULL)
 	    break;
 	}
@@ -2134,6 +2215,10 @@ open_helper(const char *instring, const char *fn, const char *mode)
   if (pipe(pfd) == -1)
     goto oops;
 
+#ifdef __EMX__
+  setmode(pfd[0], O_BINARY);
+  setmode(pfd[1], O_BINARY);
+#endif
   if (*mode == 'r')
     {
       fp = fdopen(pfd[0], "r");
@@ -2153,6 +2238,7 @@ open_helper(const char *instring, const char *fn, const char *mode)
     if ((ofd = open(ofil, O_WRONLY | O_TRUNC | O_CREAT)) == -1)
       goto oops;
 
+#ifndef __EMX__
   switch (pid = fork())
     {
     case -1:
@@ -2188,6 +2274,33 @@ open_helper(const char *instring, const char *fn, const char *mode)
       else
 	close(pfd[0]);
     }
+#else
+   {
+      int flag, tfd0, tfd1;
+      flag = fcntl(pfd[0], F_GETFD);
+      fcntl(pfd[0], F_SETFD, flag | FD_CLOEXEC);
+      flag = fcntl(pfd[1], F_GETFD);
+      fcntl(pfd[1], F_SETFD, flag | FD_CLOEXEC);
+      tfd0 = dup(0);
+      tfd1 = dup(1);
+      if (*mode == 'r')
+	dup2(pfd[1], 1);
+      if (*mode == 'w')
+	dup2(pfd[0], 0);
+      pid = spawnv(P_NOWAIT, vec[0], vec);
+      if (pid != -1) hpid = pid;
+      dup2(tfd0, 0);
+      dup2(tfd1, 1);
+      close(tfd0);
+      close(tfd1);
+      if (ofd != -1)
+	close(ofd);
+      if (*mode == 'r')
+	close(pfd[1]);
+      else
+	close(pfd[0]);
+   }
+#endif
   for (vn = 0; vn < 16; vn++)
     if (vec[vn])
       free(vec[vn]);
