@@ -5,6 +5,10 @@
 #include "gdk_imlib_private.h"
 #include <locale.h>
 
+#ifdef __EMX__
+extern const char *__XOS2RedirRoot(const char *);
+#endif
+
 char                x_error;
 
 static void
@@ -229,9 +233,17 @@ gdk_imlib_init()
   g_snprintf(s, sizeof(s), "%s/.imrc", homedir);
   old_locale = g_strdup(setlocale(LC_NUMERIC, NULL));
   setlocale(LC_NUMERIC, "C");
+#ifndef __EMX__
   f = fopen(s, "r");
+#else
+  f = fopen(s, "rt");
+#endif
   if (!f)
+#ifndef __EMX__
     f = fopen(SYSTEM_IMRC, "r");
+#else
+    f = fopen(__XOS2RedirRoot(SYSTEM_IMRC), "rt");
+#endif
   if (f)
     {
       while (fgets(s, sizeof (s), f))
@@ -719,10 +731,18 @@ gdk_imlib_init_params(GdkImlibInitParams * p)
   
   old_locale = g_strdup(setlocale(LC_NUMERIC, NULL));
   setlocale(LC_NUMERIC, "C");
-  
+
+#ifndef __EMX__  
   f = fopen(s, "r");
+#else
+  f = fopen(s, "rt");
+#endif
   if (!f)
+#ifndef __EMX__
     f = fopen(SYSTEM_IMRC, "r");
+#else
+    f = fopen(__XOS2RedirRoot(SYSTEM_IMRC), "rt");
+#endif
   if (f)
     {
       while (fgets(s, sizeof (s), f))
@@ -1316,5 +1336,9 @@ gdk_imlib_get_sysconfig()
 {
   if (!id->x.disp)
     return NULL;
+#ifndef __EMX__
   return strdup(SYSTEM_IMRC);
+#else
+  return strdup(__XOS2RedirRoot(SYSTEM_IMRC));
+#endif
 }
